# HMSHIFT - Use multispec sky emission line positions to shift wavelength
# June 3, 2005
# By Doug Mink, SAO Telescope Data Center

procedure hmshift (images)

string	images=""	{prompt="Multispec spectra for which to shift wavelength"}

string	shiftlines="1"	{prompt="Line(s) to shift in position file"}
file	emlines="hectosky1.dat" {prompt="File of emission line positions"}
file	linedir="rvsao$lib/" {prompt="Directory for emission line positions"}
string	aprange="1-300"	{prompt="Apertures over which to average pixel shift"}
bool	saveold = yes	{prompt="Save original files in Noshift/ (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print shifts (y or n)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

struct	*list1

begin

file	templist, tempemsao
string	imin, ims, history, tchar, lines, nextchar
string	filetemp, arg, temp
real	shift, wl1, shift1, shift2, shift3, shift4
bool	vb, db, firstfile, save
int	nlines, nextline

	rvsao (motd=no)
	wcstools (motd=no)

	vb = verbose
	db = debug
	ims = images
	save = saveold

	if (ims == "") {
	    print ("HMSHIFT: Find position of night sky line and shift spectra to match")
	    print ("Usage: hmshift <input multispec spectra>")
	    print ("Usage: hmshift @<input multispec list file>")
	    return
	    }

#  Set up GETCOL argument to extract line offsets from EMSAO output
	lines = ""
	nlines = 0
	i = 1
	nextchar = substr(shiftlines,i,1)
	if (nextchar != "") {
	    nextline = int (nextchar) + 8
	    lines = nextline
	    nlines = nlines + 1
	    i = i + 1
	    }
	nextchar = substr(shiftlines,i,1)
	if (nextchar == ",") {
	    i = i + 1
	    nextchar = substr(shiftlines,i,1)
	    }
	if (nextchar != "") {
	    nextline = int (nextchar) + 8
	    lines = lines // nextline
	    nlines = nlines + 1
	    i = i + 1
	    }
	nextchar = substr(shiftlines,i,1)
	if (nextchar == ",") {
	    i = i + 1
	    nextchar = substr(shiftlines,i,1)
	    }
	if (nextchar != "") {
	    nextline = int (nextchar) + 8
	    lines = lines // nextline
	    nlines = nlines + 1
	    i = i + 1
	    }
	nextchar = substr(shiftlines,i,1)
	if (nextchar == ",") {
	    i = i + 1
	    nextchar = substr(shiftlines,i,1)
	    }
	if (nextchar != "") {
	    nextline = int (nextchar) + 8
	    lines = lines // nextline
	    nlines = nlines + 1
	    }

#  Set up emsao parameters
	emsao.specnum = aprange
	emsao.specband = ""
	emsao.skynum = ""
	emsao.skyband = 0
	emsao.specdir = ""
	emsao.linefit = yes
	emsao.fixbad = no
	emsao.badlines = "badlines.dat"
	emsao.renormalize = no
	emsao.st_lambda = INDEF
	emsao.end_lambda = INDEF
	emsao.nsmooth = 0
	emsao.vel_init = "guess"
	emsao.czguess = 0.
	emsao.cortemp = ""
	emsao.wspan = 10.
	emsao.linesig = 1.
	emsao.emsearch = "skysearch.dat"
	emsao.emlines = "hectosky.dat"
	emsao.linedir = "rvsao$lib"
	emsao.npfit = 2
	emsao.nlcont = 1
	emsao.esmooth = 2
	emsao.emcombine = ""
	emsao.mincont = 0.
	emsao.lwmin = 0.4
	emsao.lwmax = 1.7
	emsao.lsmin = 1.
	emsao.sigline = 0.
	emsao.disperr = 0.01
	emsao.vel_corr = "none"
	emsao.report_mode = 9
	emsao.archive = no
	emsao.save_vel = no
	emsao.verbose = vb
	emsao.device = "stdgraph"
	emsao.hardcopy = no
	emsao.displot = db
	emsao.plotter = "stdplot"
	emsao.dispmode = 2
	emsao.vel_plot = "emission"
	emsao.curmode = no
	emsao.dispem = yes
	emsao.dispabs = no
	emsao.ablines = "ablines.dat"
	emsao.obj_plot = no
	emsao.contsub_plot = no
	emsao.debug = no
	emsao.nsum = 1
	emsao.cursor = ""

#  Make temporary list of input multispec images
	tchar = substr (ims,1,1)
	if (tchar == '@') {
	    templist = substr (ims,2,100)
	    }
	else {
            templist = mktemp ("shiftlist")
            files (ims, >templist)
	    }

# Read files to be shifted, one at a time
        list = templist
        while (fscan (list, imin) != EOF) {

# Make sure input is a FITS multispec file
	    smatch (imin, ".ms")
	    if (!smatch.found) {
		smatch (imin, ".fits")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    imin = substr (imin, 1, lroot) // ".ms.fits"
		    }
		else {
		    imin = imin // ".ms.fits"
		    }
		if (!access (imin)) {
		    print ("HMSHIFT: ", imin," does not exist")
		    next
		    }
		}

# Do not shift spectra twice
	    imgetp.default = "0"
	    imgetp (imin, "HMSHIFT")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HMSHIFT: ", imin, " has already been processed")
		    print ("HMSHIFT = ", imgetp.value)
		    }
		next
		}

#	Compute shift of selected line using emsao
            tempemsao = mktemp ("shiftemsao")
	    if (db) {
		emsao.logfiles = "STDOUT," // tempemsao
		}
	    else {
		emsao.logfiles = tempemsao
		}
	    if (debug) {
		dpar emsao
		}
#	    print ("HMSHIFT: emsao -> ",tempemsao)
	    emsao (imin)

# Compute mean wavelength shift for selected emission line and read it back
	    arg = "-mp " // tempemsao // lines
	    if (nlines == 1) {
		getcol (arg) | scan (shift)
		}
	    else if (nlines == 2) {
		getcol (arg) | scan (shift1, shift2)
		shift = (shift1 + shift2) / 2.0
		}
	    else if (nlines == 3) {
		getcol (arg) | scan (shift1, shift2, shift3)
		shift = (shift1 + shift2 + shift3) / 3.0
		}
	    else if (nlines == 4) {
		getcol (arg) | scan (shift1, shift2, shift3, shift4)
		shift = (shift1 + shift2 + shift3 + shift4) / 4.0
		}

# Save original file, if requested
	    if (save) {
		if (!access ("Noshift/")) {
		    mkdir ("Noshift")
		    }
		newfile = "Noshift/" // imin
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		copy (imin, "Noshift")
		if (vb) {
		    print ("HMSHIFT: Original ", imin, " => Noshift/")
		    }
		}

# Add shift to WCS in header of this file
	    imgetp.default = "0"
	    imgetp (imin, "CRVAL1")
	    wl1 = real (imgetp.value)
	    wl1 = wl1 - shift
	    firstfile = no
	    gettime ()
	    history = gettime.date // " " // gettime.time // " " // shift // " Angstrom shift from " // aprange
	    if (vb) {
		print ("HMSHIFT: ", history)
		}
	    hedit (imin, "CRVAL1", wl1, update=yes, verify=no, show=db)
	    hedit (imin, "HMSHIFT", history, add=yes, verify=no, show=db)

# Delete temporary file for this image
	    if (deltemp) {
		delete (tempemsao)
		}
	    }

	if (deltemp && tchar != '@') {
	    delete (templist)
	    }
end

# Jun  3 2005	Compute night sky spectrum shifts from calibrated spectrum
