# File hectospec/Scripts/hskyfilt.cl
# By Doug Mink, after Bill Wyatt
# July 23, 2007

procedure hskyfilt (skylist)

string skylist="skytest"	{prompt="File of file names to test"}

real skycert=4.0	{prompt="Max R value for certified sky"}
real skystdlim=0.2	{prompt="Max std of difference from average sky"}
string abtemps="fabtemp97,habtemp90,m31_a_temp,m31_f_temp,m31_k_temp"	{prompt="Absorption line templates for sky testing"}
string emtemps="femtemp97"	{prompt="Emission line templates for sky testing"}
bool deltemp=yes	{prompt="Delete temporary files (y or n)"}
bool verbose=yes	{prompt="note progress"}
bool debug=no		{prompt="note progress in detail"}

begin
	bool vb, dt
	string vstring, inlist
	file newlist, skysub, skyavg

	if (!deftask ("xcsao")) {
	    rvsao (motd=no)
	    }
	dt = deltemp
	vb = verbose

# Make temporary file for edited list
	newlist = mktemp ("skytemp")

# Make temporary files for mean and sky-divided spectra
	skysub = mktemp ("skysub") // ".fits"
	skyavg = mktemp ("skyav") // ".fits"
	inlist = "@" // skylist

# Compute RMS of individual skys against the sum of them; test for rejection
	if (access (newlist))
	    delete (newlist)
	for (i = 0; i < 3; i = i + 1) {
	    if (access (skyavg))
		delete (skyavg)
	    imcombine (inlist, skyavg, rejmask="", combine="average",
		       reject="avsigclip", outtype="real", scale="none",
		       zero="none",weight="!EXPTIME", mclip=yes, lsigma=3.,
		       hsigma=3., sigscale=0.1,logfile="")
	    list = skylist
	    while (fscan (list, imsky) != EOF) {
		imarith (imsky, "/", skyavg, tmpsky)
		istat (tmpsky, fields="mean,stddev")
		vstring = istat.mean
		hedit (imsky, "SKYMEAN", vstring, add+, verify-)
		vstring = istat.stddev
		hedit (imsky, "SKYSTDEV", vstring, add+, verify-)
		if (istat.stddev > skystdlim) {
		    vstring = "stdev " // vstring // " > " // skystdlim
		    hedit (imsky, "SKYREJ", vstring, add+, verify-)
		    print ("imsky"," rejected: ",vstring)
		    }
		else
		    file (imsky, >>newskylist)
		}
	    delete (skylist)
	    rename (newskylist, skylist)
	    }

# Check sky spectra for star and galaxy absorption lines
	list = skylist
	while (fscan (list, imsky) != EOF) {
	    if (access (skysub))
		delete (skysub)
	    imarith (imsky, "/", skyavg, skysub)
	    xcsaop (skysub, templates=abtemps)
	    vstring = xcsaop.besttemp // " " // xcsao.velocity // " " // xcsaop.r
	    hedit (imsky, "SKYXCABS", vstring, add+, verify-)
	    rab = xcsaop.r

# Check sky spectra for star and galaxy emission lines
	    xcsaop (skysub, templates=emtemps)
	    rem = xcsaop.r
	    vstring = rem
	    hedit (imsky, "SKYXCEM", vstring, add+, verify-)

# If either R value exceeds limit, drop sky from list
	    if (rem > skycert || rab > skycert) {
		if (rem > skycert)
		    vstring = rem
		    vstring = "Em R " // vstring // " > " // skycert
		    hedit (imsky, "SKYREJ", vstring, add+, verify-)
		    print ("imsky"," rejected: ",vstring)
		    }
		if (rab > skycert)
		    vstring = rab
		    vstring = "Abs R " // vstring // " > " // skycert
		    hedit (imsky, "SKYREJ", vstring, add+, verify-)
		    print ("imsky"," rejected: ",vstring)
		    }
		}
	    else
		file (imsky, >>newskylist)
	    }
	delete (skylist)
	rename (newskylist, skylist)
	if (deltemp) {
	    delete (skysub)
	    delete (skyavg)
	    }
end
