# File hectospec/Scripts/hskyproc.cl
# By Doug Mink, after Bill Wyatt's skyproc.ksh
# September 7, 2007

procedure hskyproc (spectra)

string spectra="\*.fits"	{prompt="Files to process"}

string skyspectra=""		{prompt="Sky spectra to combine"}
string	destdir="skysub/"	{prompt="Output dir for sky-subtracted files"}
int	skychip=0	{prompt="Chip to reduce (1 or 2, 0=both)"}
string	skymethod="skyfit"	{prompt="Sky removal method"}
string	skyline="OH8399.dat"	{prompt="Sky line to use for scaling factor"}
file	linedir="hectospec$lib/"	{prompt="Directory for sky line data"}
bool	usearea=no		{prompt="Use sky line area vs. height in scaling"}
string	skysort="skydist"	{prompt="Find closest skydistances or skyapertures"}
bool	skysame=yes	{prompt="Use skies only from same chip"}
int	nskies=6		{prompt="Max number of skies to coadd"}
real	fiberdist=0.0	{prompt="Max Fiber distance to sky"}
real	skystdlim=0.2	{prompt="Max std of difference from average sky"}
real	skycert=4.0	{prompt="Max R value for certified sky"}
string	abtemps="fabtemp97,habtemp90,m31_a_temp,m31_f_temp,m31_k_temp"	{prompt="Absorption line templates for sky testing"}
string	emtemps="femtemp97"	{prompt="Emission line templates for sky testing"}
string	logs="hskyproc.results"	{prompt="Name of output summary file"}
bool	deltemp=yes	{prompt="Delete temporary files (y or n)"}
bool	verbose=yes	{prompt="Note progress"}
bool	debug=no		{prompt="Note progress in detail"}

begin
    bool vb, dt, db
    real fiber_x, fiber_y, fiber_radius, fiberlim
    int aperture, chipnum, fiber, nsky, nsky1, nsky2, schip
    file objdb, objdb1, objdb2, skydb, skydb1, skydb2
    file oldpars, pfile, skydir
    string imagetyp, object, imf, sspec, ospec, skyfiles
    string arg0, arg, skylist1, skylist2, skylist, newlist

    dt = deltemp
    vb = verbose
    db = debug
    schip = skychip
    ospec = spectra
    sspec = skyspectra

# limit of non-parked fibers
    fiberlim = 306.0
    if (vb) {
	time()
	print ("HSKYPROC:  Sky removal method = ",skymethod)
	print ("HSKYPROC:  FiberRadiusLimit = ", fiberlim)
	print ("HSKYPROC:  Sky sorting type = ", skysort)
	print ("HSKYPROC:  Combine up to ",nskies," sky spectra")
	print ("HSKYPROC:  FiberDist < ", fiberdist, ", R < ",skycert)
	print ("HSKYPROC:  SkyStdLim = ", skystdlim)
	if (skysame)
	    print ("HSKYPROC: Skys on same chip")
	else
	    printf ("HSKYPROC: Skys on either chip")
	if (schip > 0)
	    print ("HSKYPROC: Removing background from spectra on chip ",schip)
	if (skymethod == "skysub") {
	    if (usearea)
		print ("HSKYPROC:  Sky line ", skyline," area used for scale")
	    else
		print ("HSKYPROC:  Sky line ", skyline," height used for scale")
	    }
	else
	    print ("HSKYPROC:  Sky fit to object+sky spectrum")
	print ("HSKYPROC: Destination Directory = ", destdir)
	}

# Make sure output directory is there
    if (!access (destdir)) {
	mkdir (destdir)
	}

# Set parameters for IRAF tasks used by HSKYPROC
    hsktask.oldpars = ""
    hsktask.deltemp = dt
    hsktask.verbose = vb
    hsktask ("imarith")
    hsktask ("imcombine")
    hsktask ("imdelete")
    hsktask ("istat")

# Set parameters for RVSAO tasks used by HSKYPROC
    if (!defpac ("rvsao"))
	rvsao (motd=no)
    hsktask ("pxcsao")
    hsktask ("pemsao")
    hsktask ("contpars")

# Set parameters for SVDFIT task used by HSKYPROC
    if (skymethod == "skyfit") {
	if (!defpac ("svdfit"))
	    svdfit
	hsktask ("svdres")
	}

    oldpars = hsktask.oldpars
    if (vb)
	print ("HSKYPROC: Past parameter values saved to ",oldpars)

    hskylist.skyspectra = sspec
    hskylist.skysame = skysame
    hskylist.destdir = destdir
    hskylist.skychip = schip
    hskylist.deltemp = deltemp
    hskylist.verbose = verbose
    hskylist.debug = debug
    hskylist (ospec)

# Return if no object or sky spectra in directory
    nsky = hskylist.nsky
    if (nsky <= 0) {
	print ("HSKYPROC: No sky spectra in this directory.")
	exit -1
	}
    if (hskylist.nobj <= 0) {
	print ("HSKYPROC: No object spectra in this directory.")
	exit -1
	}

# Set to single chip processing if skysame and objects only on 1 chip
    if (skysame) {
	if (hskylist.nobj1 > 0 && hskylist.nobj2 < 1) {
	    schip = 1
	    if (vb) {
		print ("HSKYPROC: No objects on chip 2; processing only chip 1")
		}
	    }
	else if (hskylist.nobj2 > 0 && hskylist.nobj1 < 1) {
	    schip = 2
	    if (vb) {
		print ("HSKYPROC: No objects on chip 1; processing only chip 2")
		}
	    }
	}

# Reject bad skies by comparing RMS of individual skys against the sum
# and checking R values by correlating sky spectra against templates
    hskyfilt.skycert = skycert
    hskyfilt.skystdlim = skystdlim
    hskyfilt.abtemps = abtemps
    hskyfilt.emtemps = emtemps
    hskyfilt.deltemp = deltemp
    hskyfilt.verbose = verbose
    hskyfilt.debug = debug

# Parameters for finding matching sky spectra, combining and removing them
    hskyfind.skymethod = skymethod
    hskyfind.skysort = skysort
    hskyfind.nskies = nskies
    hskyfind.fiberdist = fiberdist
    hskyfind.destdir = destdir
    hskyfind.verbose = verbose
    hskyfind.debug = debug

# Test skies on each chip separately
    if (skysame) {
	objdb1 = hskylist.objdb1
	objdb2 = hskylist.objdb2
	skydb1 = hskylist.skydb1
	skydb2 = hskylist.skydb2
	nsky1 = 0
	if (schip < 2) {
	    hskyfilt (skydb1, hskylist.nsky1)
	    nsky1 = hskyfilt.nsky
	    if (vb)
		print ("HSKYPROC: ",nsky1," chip 1 sky spectra from ",hskylist.nsky1)
	    if (nsky1 < 1) {
		print ("HSKYPROC: No usable sky spectra on chip 1")
		skysame = no
		schip = 0
		}
	    else {
		skydb1 = hskyfilt.outlist
		}
	    }
	nsky2 = 0
	if (schip != 1) {
	    hskyfilt (skydb2, hskylist.nsky2)
	    nsky2 = hskyfilt.nsky
	    if (vb)
		print ("HSKYPROC: ",nsky2," chip 2 sky spectra from ",hskylist.nsky2)
	    if (nsky2 < 1) {
		print ("HSKYPROC: No usable sky spectra on chip 2")
		skysame = no
		hskyfilt (skydb1, insky=hskylist.nsky2)
		nsky1 = hskyfilt.nsky
		if (nsky1 > 0) {
		    skydb2 = skydb1
		    }
		}
	    else {
		skydb2 = hskyfilt.outlist
		if (nsky1 < 1) {
		    skydb1 = skydb2
		    }
		}
	    }
	nsky = nsky1 + nsky2

# Combine closest sky spectra for each object spectrum and remove sky
 	if (nsky < 1)
	    print ("HSKYPROC: No usable sky spectra in this directory")
	else {
	    if (schip < 2) {
		if (db)
		    print ("HSKYPROC: Running HSKYFIND on chip 1 data")
		hskyfind (objdb1, skydb1)
		}
	    if (schip != 1) {
		if (db)
		    print ("HSKYPROC: Running HSKYFIND on chip 2 data")
		hskyfind (objdb2, skydb2)
		}
	    }
	}

# Use all skies on chip for every object
    else {
	skydb = hskylist.skydb
	objdb = hskylist.objdb
	if (access (skydb)) {
	    wc (skydb2) | scan (nsky2)
	    }
	else {
	    hskyfilt (skydb, insky=hskylist.nsky)
	    nsky = hskyfilt.nsky
	    }

 	if (nsky < 1)
	    print ("HSKYPROC: No usable sky spectra in this directory")

# Combine closest sky spectra for each object spectrum and remove sky
	hskyfind (objdb, skydb)
	}

# Save parameter files
	pfile = destdir // "*.par"
	delete (pfile)
	pfile = destdir // "hskyproc.par"
	dpar ("hskyproc", >pfile)
	pfile = destdir // "istat.par"
	dpar ("istat", >pfile)
	pfile = destdir // "imarith.par"
	dpar ("imarith", >pfile)
	pfile = destdir // "imcombine.par"
	dpar ("imcombine", >pfile)
	pfile = destdir // "imdelete.par"
	dpar ("imdelete", >pfile)
	pfile = destdir // "pemsao.par"
	dpar ("rvsao.pemsao", >pfile)
	pfile = destdir // "pxcsao.par"
	dpar ("rvsao.pxcsao", >pfile)
	pfile = destdir // "contpars.par"
	dpar ("rvsao.contpars", >pfile)

	if (skymethod == "skyfit") {
	    pfile = destdir // "svdres.par"
	    dpar ("svdres", >pfile)
	    }

# Move sky-subtracted sky files to skydir subdirectory
	skydir = destdir // "skydir/"
	if (!access (skydir))
	    mkdir (skydir)
	skyfiles = destdir // "*.sky.ms.fits"
	move (skyfiles, skydir)

# Reset parameters to original values
    cl (<oldpars)
    if (dt)
	delete (oldpars)
    if (vb) {
	print ("HSKYPROC: Past parameter values reset from ",oldpars)
	time ()
	}
end

# Aug 31 2007	New task
# Sep  6 2007	Add skyspectra to subtract but not process skies
# Sep  7 2007	Set to single chip if skysame and objects only on one chip
