# HTRAN -- Correct for fiber throughput differences
# By Doug Mink, SAO/TDC
# May 22, 2007

procedure htran (images)

string	images = ""	{prompt="Input multispec images to correct"}
string	skycorr=""	{prompt="Transmission from sky flat via maketran"}
string  skyfile="hectospec$lib/skyabs.ms.fits"  {prompt="Sky absorption spectrum normalized to continuum=1"}
bool deltemp = no	{prompt="Delete temporary files (y or n)"}
bool saveold = yes	{prompt="Save unprocessed files to Notran/ (y or n)"}
bool verbose = yes	{prompt="Log progress (y or n)"}
bool debug = no		{prompt="Log each aperture (y or n)"}

begin
	bool vb, db, save, del
	string	imtype, mstype, keyroot, apxkey, apykey, rratio, aps
	file imin, imout, imcorr, geores, geotemp, imroot, newfile, templist
	file imsky
	string imageap, history, order, function, ims, tchar
	int index, iap, nap, npix, lroot, xcol, ycol
	real x, y, rad, ratio

	if (!deftask ("emsao")) {
	    rvsao (motd=no)
	    }
	if (!defpac ("wcstools")) {
	    wcstools (motd=no)
	    }

	ims = images
	imcorr = skycorr
	imsky = skyfile
	vb = verbose
	db = debug
	del = deltemp
	save = saveold

# Make sure throughput correction filename ends in .ms.fits
	if (imcorr == "") {
	    print ("HTRAN: no throughput correction file given")
	    return
	    }
	smatch (imcorr, ".ms")
	if (smatch.found) {
	    smatch (imcorr, ".fits")
	    if (!smatch.found) {
		imcorr = imcorr // ".fits"
		}
	    }
	else {
	    smatch (imcorr, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imcorr = substr (imcorr, 1, lroot) // ".ms.fits"
		}
	    else {
		imcorr = imcorr // ".ms.fits"
		}
	    }

# Check for existence of throughput correction file
	if (!access (imcorr)) {
	    print ("HTRAN: throughput correction file ",imcorr," cannot be read")
	    return
	    }

# Make sure sky absorption filename ends in .ms.fits
	if (imsky != "") {
	    smatch (imsky, ".ms")
	    if (smatch.found) {
		smatch (imsky, ".fits")
		if (!smatch.found) {
		    imsky = imsky // ".fits"
		    }
		}
	    else {
		smatch (imsky, ".fits")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    imsky = substr (imsky, 1, lroot) // ".ms.fits"
		    }
		else {
		    imsky = imsky // ".ms.fits"
		    }
		}

# Check for existence of throughput correction file
	    if (!access (imsky)) {
		print ("HTRAN: sky absorption file ",imsky," cannot be read")
		return
		}
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
        if (tchar == '@') {
	    templist = substr (ims,2,100)
	    }
	else {
	    templist = mktemp ("htranlist")
	    files (ims, >templist)
	    }
	list = templist
	while (fscan (list,imin) != EOF) {

# If geometrical flux correction has not been done, do it
#	    hgeom1 (imin, deltemp=del, saveold=save, verbose=vb, debug=db)

# If throughput correction has not been done, do it
	    htran1 (imin, skycorr=imcorr, deltemp=del, saveold=save, verbose=vb, debug=db)

# If sky absorption has not been corrected for, do it
	    if (imsky != "") {
		habsky1 (imin, skyfile=imsky, deltemp=del, saveold=save, verbose=vb, debug=db)
		}

# If red leak has not been corrected for, do it
	    hredleak1 (imin, deltemp=del, saveold=save, verbose=vb, debug=db)
	    }

	if (deltemp && tchar != '@') {
	    delete (templist)
	    }
end

# Jun 10 2004	New task
# Sep  2 2004	Handle multiple files in one run
# Sep 23 2004	Change "transmission" to "throughput"
# Oct 18 2004	Change FIBRTRAN keyword to HTRAN
# Oct 18 2004	Print value of HTRAN/FIBRTRAN if already processed
# Oct 18 2004	Split off geometric correction to HGEOM task

# Jan 10 2005	Improve input filename options
# Jan 13 2005	Fix typo in temporary list variable name on deletion
# Mar  9 2005	Call HGEOM1 to do geometric correction, not HGEOM
# Mar  9 2005	Move computation to single image HTRAN1
# Mar 17 2005	Do not turn off deltemp if reading from file list
# May 31 2005	Add HABSKY1 to remove sky absorption
# Jun  9 2005	Add HREDLEAK1 to remove red leak from spectra
# Aug  8 2005	Drop out of program if throughput correction file is not given
# Aug  8 2005	Drop out of program if throughput correction file cannot be read
# Aug  8 2005	Skip sky correction if sky absorption file is not given
# Aug  8 2005	Drop out of program if sky absorption file cannot be read
# Aug 10 2005	Load wcstools and rvsao only if they are not already loaded

# Aug 23 2006	Drop geometric connection; it should only be used for sky

# May 22 2007	Check for package wcstools instead of task sethead
