# File t_hskymatch.x
# August 29, 2007
# By Doug Mink

include <error.h>
include <ctype.h>
include <imhdr.h>
include <imio.h>
include <mach.h>

define	MAXSKY	300
define	SKY_SORTS	"|aperture|skydist|"
define	SORT_AP		1
define	SORT_DIST	2

# HSKYMATCH -- Make a list of sky spectra which meet the given
# distance standard for subtraction from an object+sky spectrum

procedure t_hskymatch ()

char	objfile[SZ_FNAME]
int	objap
double	objfibx, objfiby
char	skylist[SZ_FNAME]
double	maxdist
bool	vb, db
char	skysort[SZ_LINE]
char	skyfile[SZ_FNAME,MAXSKY]
char	skyfiles[SZ_PATHNAME]
char	skyaps[SZ_LINE]
char	tempap[8]
int	skydex[MAXSKY]
int	skyap[MAXSKY]
int	skyfib[MAXSKY]
double	skyfibx[MAXSKY]
double	skyfiby[MAXSKY]
int	i, maxskies, isky, fd, nuse
int	sortsky
double	dx,dy
int	fscan(), open()
extern	compsky()
extern	compi()
int	strdic()
int	clgeti()
double	clgetd() 
bool	clgetb()
long	clktime()
double	dist[MAXSKY]	# table of distances between object and sky
int	nsky		# Number of sky spectra in use

begin

# Get parameters

# Filename of object plus sky spectrum
	call clgstr ("objfile", objfile, SZ_FNAME)

# File name of table of possible sky spectra with ap, fiber position
	call clgstr ("skylist", skylist, SZ_FNAME)

# Object aperture
	objap = clgeti ("objap")

# Object fiber X coordinate
	objfibx = clgetd ("objfibx")

# Object fiber Y coordinate
	objfiby = clgetd ("objfiby")

# Type of distance sorting (aperture or focal plane distance)
	call clgstr ("skysort", skysort, SZ_LINE)
	sortsky = strdic (skysort, skysort, SZ_LINE, SKY_SORTS)

# Maximum number of sky spectra to return
	maxskies = clgeti ("maxskies")

# Maximum focal plane or aperture distance for sky spectra
	maxdist = clgetd ("maxdist")

	vb = clgetb ("verbose")
	db = clgetb ("debug")

# Read the sky information from the input file
	nsky = 0
	fd = open (skylist, READ_ONLY, TEXT_FILE)
	if (fd < 3) {
	    call clputi ("nsky", 0)
	    call clpstr ("skyfiles", "")
	    call clpstr ("skyaps", "")
	    call printf ("HSKYMATCH: Cannot open sky list file %s\n")
		call pargstr (skylist)
	    call flush (STDOUT)
	    return
	    }

	if (db) {
	    call printf ("HSKYMATCH: Reading from sky list file %s\n")
		call pargstr (skylist)
	    call flush (STDOUT)
	    }
# Skip first two lines of headings
	if (fscan(fd) != EOF) {
	    if (fscan(fd) != EOF) {
		}
	    }
	while (fscan(fd) != EOF && nsky < MAXSKY) {
	    nsky = nsky + 1
#	    if (db) {
#		call printf ("%3d\n")
#		    call pargi (nsky)
#		call flush (STDOUT)
#		}
	    call gargwrd (skyfile[1,nsky], SZ_FNAME)
	    call gargi (skyap[nsky])
	    call gargi (skyfib[nsky])
	    call gargd (skyfibx[nsky])
	    call gargd (skyfiby[nsky])

# Tabulate the distance of the sky spectrum from the object
	    if (sortsky == SORT_DIST) {
		do i = 1, nsky {
		    dx = objfibx - skyfibx[nsky]
		    dy = objfiby - skyfiby[nsky]
		    dist[nsky] = sqrt (dx * dx + dy * dy)
		    }
		}
	    else {
		do i = 1, nsky {
		    dx = objap - skyap[nsky]
		    dist[nsky] = sqrt (dx * dx)
		    }
		}

# Make an index array for sorting
	    skydex[nsky] = nsky

	    if (db) {
		call printf ("%3d\t%s\t%3d\t%3d\t%7.3f\t%7.3f\t%7.3f\n")
		    call pargi (skydex[nsky])
		    call pargstr (skyfile[1,nsky])
		    call pargi (skyap[nsky])
		    call pargi (skyfib[nsky])
		    call pargd (skyfibx[nsky])
		    call pargd (skyfiby[nsky])
		    call pargd (dist[nsky])
		call flush (STDOUT)
		}
	    }
	call close (fd)

	if (db) {
	    call printf ("HSKYMATCH: Read in %d sky spectra\n")
		call pargi (nsky)
	    call flush (STDOUT)
	    }

	if (nsky < 1) {
	    call clputi ("nsky", 0)
	    call clpstr ("skyfiles", "")
	    call clpstr ("skyaps", "")

	    if (vb) {
		call printf ("HSKYMATCH: No skies were found for %s")
		    call pargstr (objfile)
		call flush (STDOUT)
		}
	    return
	    }

	if (db) {
	    call printf ("HSKYMATCH: About to sort %d sky spectra\n")
		call pargi (nsky)
	    call flush (STDOUT)
	    }

	call gqsort (skydex, nsky, compsky, dist)

	if (db) {
	    call printf ("HSKYMATCH: Finished sorting %d sky spectra\n")
		call pargi (nsky)
	    call flush (STDOUT)
	    }

	if (maxskies < nsky)
	    nsky = maxskies
	if (maxdist > 0.0) {
	    nuse = 0
	    do i = 1, nsky {
		if (dist[skydex[i]] > maxdist)
		    break
		nuse = nuse + 1
		}
	    }
	else
	    nuse = nsky

# If there are no usable sky spectra, set parameters and return
	if (nuse < 1) {
	    call clputi ("nsky", 0)
	    call clpstr ("skyfiles", "")
	    call clpstr ("skyaps", "")

	    if (vb) {
		call printf ("HSKYMATCH: No acceptable skies were found for %s")
		    call pargstr (objfile)
		}
	    return
	    }

# Otherwise, return lists of sky apertures and files
	else {
	    nsky = nuse

# Sort numerically so combination sky file names are same for same aps
	    call gqsort (skydex, nsky, compi, dist)
	    if (vb) {
		call printf ("HSKYMATCH: %d acceptable skies were found for %s\n")
		    call pargi (nsky)
		    call pargstr (objfile)
		}
	    do i = 1, nsky {
		isky = skydex[i]
		if (i == 1) {
		    call sprintf (tempap,7,"%d")
			call pargi (skyap[isky])
		    call strcpy (tempap, skyaps)
		    call strcpy (skyfile[1,isky], skyfiles)
		    }
		else {
		    call sprintf (tempap,7,"_%d")
			call pargi (skyap[isky])
		    call strcat (tempap, skyaps)
		    call strcat (",", skyfiles)
		    call strcat (skyfile[1,isky], skyfiles)
		    }
		}
	    call clputi ("nsky", nsky)
	    call clpstr ("skyfiles", skyfiles)
	    call clpstr ("skyaps", skyaps)
	    if (vb) {
		call printf ("HSKYMATCH: Sky files: %s\n")
		    call pargstr (skyfiles)
		}
	    }
	return
end


# Sort subroutine returns 1 if dist[i]>dist[j], -1 if dist[i]<dist[j], else 0

int procedure compsky (dist, i, j)

double	dist[MAXSKY]	# table of distances between object and sky
int i, j		# Array indices to be compared.

begin
	if (dist[i] < dist[j])
	    return (-1)
	else if (dist[i] > dist[j])
	    return (1)
	else
	    return (0)
end


# Sort subroutine returns 1 if i>j, -1 if i<j, else 0

int procedure compi (dist, i, j)

double	dist[MAXSKY]	# table of distances between object and sky
int i, j		# Array indices to be compared.

begin
	if (i < j)
	    return (-1)
	else if (i > j)
	    return (1)
	else
	    return (0)
end

# Aug 29 2007	New program
